<?php
/**
 * Theme Init Functions
 * @package laevento
 * @since 1.0.0
 */

if (!defined("ABSPATH")) {
    exit(); //exit if access directly
}

if (!class_exists('Laevento_Init')) {

    class Laevento_Init
    {
       /**
        * $instance
        * @since 1.0.0
        */
        protected static $instance;

        public function __construct()
        {
            /*
             * theme setup
             */
            add_action('after_setup_theme', array($this, 'theme_setup'));
            /**
             * Widget Init
             */
            add_action('widgets_init', array($this, 'theme_widgets_init'));
            /**
             * Theme Assets
             */
            add_action('wp_enqueue_scripts', array($this, 'theme_assets'));
            /**
             * Registers an editor stylesheet for the theme.
             */
            add_action('admin_init', array($this, 'add_editor_styles'));
        }

        /**
         * getInstance()
         */
        public static function getInstance()
        {
            if (null == self::$instance) {
                self::$instance = new self();
            }
            return self::$instance;
        }

        /**
         * Theme Setup
         * @since 1.0.0
         */
        public function theme_setup()
        {
            /*
             * Make theme available for translation.
             * Translations can be filed in the /languages/ directory.
             */
            load_theme_textdomain('laevento', get_template_directory() . '/languages');

            // Add default posts and comments RSS feed links to head.
            add_theme_support('automatic-feed-links');

            /*
             * Let WordPress manage the document title.
             */
            add_theme_support('title-tag');

            /*
             * Enable support for Post Thumbnails on posts and pages.
             *
             * @link https://developer.wordpress.org/themes/functionality/featured-images-post-thumbnails/
             */
            add_theme_support('post-thumbnails');

            // This theme uses wp_nav_menu() in one location.
            register_nav_menus(array(
                'main-menu' => esc_html__('Primary Menu', 'laevento'),
            ));

            /*
             * Switch default core markup for search form, comment form, and comments
             * to output valid HTML5.
             */
            add_theme_support('html5', array(
                'search-form',
                'comment-form',
                'comment-list',
                'gallery',
                'caption',
            ));

            // Add theme support for selective wp block styles
            add_theme_support("wp-block-styles");
            // Add theme support for selective align wide
            add_theme_support("align-wide");
            // Add theme support for selective responsive embeds
            add_theme_support("responsive-embeds");

            // Add theme support for selective refresh for widgets.
            add_theme_support('customize-selective-refresh-widgets');

            /**
             * Add support for core custom logo.
             *
             * @link https://codex.wordpress.org/Theme_Logo
             */
            add_theme_support('custom-logo', array(
                'height' => 250,
                'width' => 250,
                'flex-width' => true,
                'flex-height' => true,
            ));

            //woocommerce support
            add_theme_support('woocommerce');
            add_theme_support('wc-product-gallery-zoom');
            add_theme_support('wc-product-gallery-lightbox');
            add_theme_support('wc-product-gallery-slider');


            //add theme support for post format
            add_theme_support('post-formats', array('image', 'video', 'gallery', 'link', 'quote'));

            // This variable is intended to be overruled from themes.
            $GLOBALS['content_width'] = apply_filters('laevento_content_width', 740);

            //add image sizes
            add_image_size('laevento_classic', 750, 500, true);
            add_image_size('laevento_grid_blog', 370, 270, true);
            add_image_size('laevento_classic_event', 300, 310, true);
            add_image_size('laevento_classic_team', 280, 350, true);
            add_image_size('laevento_team_circle', 370, 370, true);
            add_image_size('laevento_event_image', 1145, 500, true);

            self::load_theme_dependency_files();
        }

        /**
         * Theme Widget Init
         * @since 1.0.0
         * @link https://developer.wordpress.org/themes/functionality/sidebars/#registering-a-sidebar
         */
        public function theme_widgets_init()
        {
            register_sidebar(array(
                'name' => esc_html__('Sidebar', 'laevento'),
                'id' => 'sidebar-1',
                'description' => esc_html__('Add widgets here.', 'laevento'),
                'before_widget' => '<div id="%1$s" class="widget %2$s">',
                'after_widget' => '</div>',
                'before_title' => '<h4 class="widget-headline style-01">',
                'after_title' => '</h4>',
            ));
            register_sidebar(array(
                'name' => esc_html__('Footer Widget Area', 'laevento'),
                'id' => 'footer-widget',
                'description' => esc_html__('Add widgets here.', 'laevento'),
                'before_widget' => '<div class="col-lg-3 col-md-6"><div id="%1$s" class="widget footer-widget %2$s">',
                'after_widget' => '</div></div>',
                'before_title' => '<h4 class="widget-headline">',
                'after_title' => '</h4>',
            ));
        }

        /**
         * Theme Assets
         * @since 1.0.0
         */
        public function theme_assets()
        {
            self::load_theme_css();
            self::load_theme_js();
        }

      /*
       * Load theme options google fonts css
       * @since 1.0.0
       */
        public static function load_google_fonts()
        {

            $enqueue_fonts = array();
            //body font enqueue
            $body_font = cs_get_option('_body_font') ? cs_get_option('_body_font') : false;
            $body_font_variant = cs_get_option('body_font_variant') ? cs_get_option('body_font_variant') : false;
            $body_font['family'] = (isset($body_font['font-family']) && !empty($body_font['font-family'])) ? $body_font['font-family'] : 'Poppins';
            $body_font['font'] = (isset($body_font['type']) && !empty($body_font['type'])) ? $body_font['type'] : 'google';
            $body_font_variant = !empty($body_font_variant) ? $body_font_variant : array(400, 700, 600, 500);
            $google_fonts = array();

            if (!empty($body_font_variant)) {
                foreach ($body_font_variant as $variant) {
                    $google_fonts[] = array(
                        'family' => $body_font['family'],
                        'variant' => $variant,
                        'font' => $body_font['font']
                    );
                }
            }
            //heading font enqueue
            $heading_font_enable = false;
            if (null == cs_get_option('heading_font_enable')) {
                $heading_font_enable = false;
            } elseif ('0' == cs_get_option('heading_font_enable')) {
                $heading_font_enable = true;
            } elseif ('1' == cs_get_option('heading_font_enable')) {
                $heading_font_enable = false;
            }
            $heading_font = cs_get_option('heading_font') ? cs_get_option('heading_font') : false;
            $heading_font_variant = cs_get_option('heading_font_variant') ? cs_get_option('heading_font_variant') : false;
            $heading_font['family'] = (isset($heading_font['font-family']) && !empty($heading_font['font-family'])) ? $heading_font['font-family'] : 'Jost';
            $heading_font['font'] = (isset($heading_font['type']) && !empty($heading_font['type'])) ? $heading_font['type'] : 'google';
            $heading_font_variant = !empty($heading_font_variant) ? $heading_font_variant : array(400, 500, 600, 700, 800);
            if (!empty($heading_font_variant) && !$heading_font_enable) {
                foreach ($heading_font_variant as $variant) {
                    $google_fonts[] = array(
                        'family' => $heading_font['family'],
                        'variant' => $variant,
                        'font' => $heading_font['font']
                    );
                }
            }

            if (!empty($google_fonts)) {
                foreach ($google_fonts as $font) {
                    if (!empty($font['font']) && $font['font'] == 'google') {
                        $variant = (!empty($font['variant']) && $font['variant'] !== 'regular') ? ':' . $font['variant'] : '';
                        if (!empty($font['family'])) {
                            $enqueue_fonts[] = $font['family'] . $variant;
                        }
                    }
                }
            }

            $enqueue_fonts = array_unique($enqueue_fonts);
            return $enqueue_fonts;
        }

        /**
         * Load Theme Css
         * @since 1.0.0
         */
        public function load_theme_css()
        {
            $theme_version = LAEVENTO_DEV ? time() : laevento()->get_theme_info('version');
            $css_ext = '.css';
            //load google fonts
            $enqueue_google_fonts = self::load_google_fonts();
            if (!empty($enqueue_google_fonts)) {
                wp_enqueue_style('laevento-google-fonts', esc_url(add_query_arg('family', urlencode(implode('|', $enqueue_google_fonts)), '//fonts.googleapis.com/css')), array(), null);
            }
            $all_css_files = array(
                array(
                    'handle' => 'animate',
                    'src' => LAEVENTO_CSS . '/animate.min.css',
                    'deps' => array(),
                    'ver' => $theme_version,
                    'media' => 'all',
                ),
                array(
                    'handle' => 'backtotop',
                    'src' => LAEVENTO_CSS . '/backToTop.css',
                    'deps' => array(),
                    'ver' => $theme_version,
                    'media' => 'all',
                ),
                array(
                    'handle' => 'bootstrap',
                    'src' => LAEVENTO_CSS . '/bootstrap.min.css',
                    'deps' => array(),
                    'ver' => $theme_version,
                    'media' => 'all',
                ),
                array(
                    'handle' => 'font-awesome',
                    'src' => LAEVENTO_CSS . '/fontawesome-all.min.css',
                    'deps' => array(),
                    'ver' => '5.12.0',
                    'media' => 'all',
                ),
                array(
                    'handle' => 'icomoon',
                    'src' => LAEVENTO_CSS . '/icomoon.css',
                    'deps' => array(),
                    'ver' => $theme_version,
                    'media' => 'all',
                ),
                array(
                    'handle' => 'glightbox',
                    'src' => LAEVENTO_CSS . '/glightbox.min.css',
                    'deps' => array(),
                    'ver' => $theme_version,
                    'media' => 'all',
                ),
                array(
                    'handle' => 'lightcase',
                    'src' => LAEVENTO_CSS . '/lightcase.css',
                    'deps' => array(),
                    'ver' => $theme_version,
                    'media' => 'all',
                ),
                array(
                    'handle' => 'magnific-popup',
                    'src' => LAEVENTO_CSS . '/magnific-popup.css',
                    'deps' => array(),
                    'ver' => $theme_version,
                    'media' => 'all',
                ),
                array(
                    'handle' => 'nice-select',
                    'src' => LAEVENTO_CSS . '/nice-select.css',
                    'deps' => array(),
                    'ver' => $theme_version,
                    'media' => 'all',
                ),
                array(
                    'handle' => 'slick',
                    'src' => LAEVENTO_CSS . '/slick.css',
                    'deps' => array(),
                    'ver' => $theme_version,
                    'media' => 'all',
                ),
                array(
                    'handle' => 'laevento-main-style',
                    'src' => LAEVENTO_CSS . '/main-style' . $css_ext,
                    'deps' => array(),
                    'ver' => $theme_version,
                    'media' => 'all',
                ),
                array(
                    'handle' => 'laevento-responsive',
                    'src' => LAEVENTO_CSS . '/responsive' . $css_ext,
                    'deps' => array(),
                    'ver' => $theme_version,
                    'media' => 'all',
                ),
            );
            $all_css_files = apply_filters('laevento_theme_enqueue_style', $all_css_files);

            if (is_array($all_css_files) && !empty($all_css_files)) {
                foreach ($all_css_files as $css) {
                    call_user_func_array('wp_enqueue_style', $css);
                }
            }
            wp_enqueue_style('laevento-style', get_stylesheet_uri());

            if (laevento()->is_laevento_core_active()) {
                if (file_exists(LAEVENTO_DYNAMIC_STYLESHEETS . '/theme-inline-css-style.php')) {
                    require_once LAEVENTO_DYNAMIC_STYLESHEETS . '/theme-inline-css-style.php';
                    require_once LAEVENTO_DYNAMIC_STYLESHEETS . '/theme-option-css-style.php';
                    wp_add_inline_style('laevento-style', laevento()->minify_css_lines($GLOBALS['laevento_inline_css']));
                    wp_add_inline_style('laevento-style', laevento()->minify_css_lines($GLOBALS['theme_customize_css']));
                }

            }
        }

        /**
         * Load Theme js
         * @since 1.0.0
         */
        public function load_theme_js()
        {
            $theme_version = laevento()->get_theme_info('version');
            $js_ext = LAEVENTO_DEV ? '.js' : '.min.js';
           

            $scripts = [
            ['handle' => 'bootstrap', 'src'=> LAEVENTO_JS . '/bootstrap.min.js','dep'=> array( 'jquery' ),'version'=> $theme_version,'in_foot'=> true],
            ['handle' => 'bootstrap-bundle', 'src'=> LAEVENTO_JS . '/bootstrap.bundle.js','dep'=> array( 'jquery' ),'version'=> $theme_version,'in_foot'=> true],
            ['handle' => 'backtotop', 'src'=> LAEVENTO_JS . '/backToTop.js','dep'=> array( 'jquery' ),'version'=> $theme_version,'in_foot'=> true],
            ['handle' => 'counterup', 'src'=> LAEVENTO_JS . '/counterup.js','dep'=> array( 'jquery' ),'version'=> $theme_version,'in_foot'=> true],
            ['handle' => 'glightbox', 'src'=> LAEVENTO_JS . '/glightbox.min.js','dep'=> array( 'jquery' ),'version'=> $theme_version,'in_foot'=> true],
            ['handle' => 'isotope', 'src'=> LAEVENTO_JS . '/isotope.pkgd.min.js','dep'=> array( 'jquery' ),'version'=> $theme_version,'in_foot'=> true],
            ['handle' => 'magnific-popup', 'src'=> LAEVENTO_JS . '/jquery.magnific-popup.min.js','dep'=> array( 'jquery' ),'version'=> $theme_version,'in_foot'=> true],
            ['handle' => 'nice-select', 'src'=> LAEVENTO_JS . '/jquery.nice-select.min.js','dep'=> array( 'jquery' ),'version'=> $theme_version,'in_foot'=> true],
            ['handle' => 'rProgressbar', 'src'=> LAEVENTO_JS . '/jQuery.rProgressbar.min.js','dep'=> array( 'jquery' ),'version'=> $theme_version,'in_foot'=> true],
            ['handle' => 'scrollUp', 'src'=> LAEVENTO_JS . '/jquery.scrollUp.min.js','dep'=> array( 'jquery' ),'version'=> $theme_version,'in_foot'=> true],
            ['handle' => 'lightcase', 'src'=> LAEVENTO_JS . '/lightcase.js','dep'=> array( 'jquery' ),'version'=> $theme_version,'in_foot'=> true],
            ['handle' => 'popper', 'src'=> LAEVENTO_JS . '/popper.min.js','dep'=> array( 'jquery' ),'version'=> $theme_version,'in_foot'=> true],
            ['handle' => 'waypoints', 'src'=> LAEVENTO_JS . '/waypoints.js','dep'=> array( 'jquery' ),'version'=> $theme_version,'in_foot'=> true],
            ['handle' => 'tweenlite', 'src'=> LAEVENTO_JS . '/tweenlite.min.js','dep'=> array( 'jquery' ),'version'=> $theme_version,'in_foot'=> true],
            ['handle' => 'wow', 'src'=> LAEVENTO_JS . '/wow.min.js','dep'=> array( 'jquery' ),'version'=> $theme_version,'in_foot'=> true],
            ['handle' => 'laevento-main-script', 'src'=> LAEVENTO_JS . '/main' . $js_ext,'dep'=> array( 'jquery' ),'version'=> $theme_version,'in_foot'=> true]
            ];

            for ($i=0; $i < sizeof($scripts); $i++) {
                wp_enqueue_script( $scripts[$i]['handle'], $scripts[$i]['src'], $scripts[$i]['dep'], $scripts[$i]['version'], $scripts[$i]['in_foot'] );    
            }

            if (is_singular() && comments_open() && get_option('thread_comments')) {
                wp_enqueue_script('comment-reply');
            }
        }

        /**
         * Load THeme Dependency Files
         * @since 1.0.0
         */
        public function load_theme_dependency_files()
        {
            $includes_files = array(
                array(
                    'file-name' => 'activation',
                    'file-path' => LAEVENTO_TGMA
                ),
                array(
                    'file-name' => 'theme-breadcrumb',
                    'file-path' => LAEVENTO_INC
                ),
                array(
                    'file-name' => 'theme-excerpt',
                    'file-path' => LAEVENTO_INC
                ),
                array(
                    'file-name' => 'theme-hook-customize',
                    'file-path' => LAEVENTO_INC
                ),
                array(
                    'file-name' => 'theme-comments-modifications',
                    'file-path' => LAEVENTO_INC
                ),
                array(
                    'file-name' => 'customizer',
                    'file-path' => LAEVENTO_INC
                ),
                array(
                    'file-name' => 'theme-group-fields-cs',
                    'file-path' => LAEVENTO_THEME_SETTINGS
                ),
                array(
                    'file-name' => 'theme-group-fields-value-cs',
                    'file-path' => LAEVENTO_THEME_SETTINGS
                ),
                array(
                    'file-name' => 'theme-metabox-cs',
                    'file-path' => LAEVENTO_THEME_SETTINGS
                ),
                array(
                    'file-name' => 'theme-userprofile-cs',
                    'file-path' => LAEVENTO_THEME_SETTINGS
                ),
                array(
                    'file-name' => 'theme-shortcode-option-cs',
                    'file-path' => LAEVENTO_THEME_SETTINGS
                ),
                array(
                    'file-name' => 'theme-customizer-cs',
                    'file-path' => LAEVENTO_THEME_SETTINGS
                ),
                array(
                    'file-name' => 'theme-option-cs',
                    'file-path' => LAEVENTO_THEME_SETTINGS
                ),
            );
            if (file_exists('WooCommerce')) {
                $includes_files[] = array(
                    'file-name' => 'class-woocommerce-customize',
                    'file-path' => LAEVENTO_INC
                );
            }
            if (is_array($includes_files) && !empty($includes_files)) {
                foreach ($includes_files as $file) {
                    if (file_exists($file['file-path'] . '/' . $file['file-name'] . '.php')) {
                        require_once $file['file-path'] . '/' . $file['file-name'] . '.php';
                    }
                }
            }


        }

        /**
         * Add editor style
         * @since 1.0.0
         */
        public function add_editor_styles()
        {
            add_editor_style(get_template_directory_uri() . '/assets/css/editor-style.css');
        }

    }//end class
    if (class_exists('Laevento_Init')) {
        Laevento_Init::getInstance();
    }
}
